<?php

/**
 * @file
 * Configuration options for the module.
 */

/**
 * Administration form.
 */
function conditional_message_config_form($form, &$form_state) {

  // Show appropriate form depending on the sub-module that takes precedence.
  if (module_exists('conditional_message_client')) {
    $form['conditional_message'] = array(
      '#title' => t('Conditional message client configurations'),
      '#type' => 'fieldset',
    );

    // Display in columns.
    drupal_add_css('.conditional-message-options-columns {column-count:2;column-gap:10px;}', 'inline');

    $form['conditional_message']['multisite_setup'] = array(
      '#title' => t('Client site setup'),
      '#type' => 'fieldset',
    );

    $server_site = variable_get('conditional_message_server_site', 'https://example.com');
    $form['conditional_message']['multisite_setup']['conditional_message_server_site'] = array(
      '#type' => 'textfield',
      '#title' => t('Server site'),
      '#description' => t('Enter the URL of the site that will provide the configurations (https://example.com).'),
      '#default_value' => $server_site,
      '#size' => 60,
      '#maxlength' => 255,
      '#required' => FALSE,
    );

    $form['#validate'][] = 'conditional_message_client_config_form_validate';
  }
  else {
    // This is the default form if no sub-module takes it's precedence.
    $form['conditional_message'] = array(
      '#title' => t('Conditional message configurations'),
      '#type' => 'fieldset',
    );

    $conditional_message_options = array(
      'session' => t('once per session'),
      'close' => t('until manually closed'),
      'role' => t('to given user roles'),
      'content_type' => t('on given content types'),
      'path' => t('on given paths'),
      'multisite' => t('on given multi-sites'),
    );

    $form['conditional_message']['conditional_message_always'] = array(
      '#type' => 'checkbox',
      '#title' => t('Always display the message'),
      '#default_value' => variable_get('conditional_message_always', 1),
    );

    $form['conditional_message']['conditional_message_options'] = array(
      '#title' => t('Conditions'),
      '#type' => 'checkboxes',
      '#description' => t('Select the conditions in which the message will be displayed. Several conditions can be selected.'),
      '#default_value' => variable_get('conditional_message_options'),
      '#required' => FALSE,
      '#options' => $conditional_message_options,
      '#attributes' => array('class' => array('conditional-message-options-columns')),
      '#states' => array(
        'invisible' => array(
          array(':input[name="conditional_message_always"]' => array('checked' => TRUE)),
        ),
      ),
    );

    // Display in columns.
    drupal_add_css('.conditional-message-options-columns {column-count:2;column-gap:10px;}', 'inline');

    $form['conditional_message']['message'] = array(
      '#title' => t('Message'),
      '#type' => 'fieldset',
    );

    $text = variable_get('conditional_message_text');
    $text = !empty($text['value']) ? $text['value'] : 'Enter a custom message at /admin/config/user-interface/conditional-message';
    $form['conditional_message']['message']['conditional_message_text'] = array(
      '#title' => t('Text'),
      '#type' => 'text_format',
      '#format' => isset($edit['format']) ? $edit['format'] : NULL, // TODO get the correct variable for the format.
      '#description' => t("Set the message to be displayed when conditions are met (HTML is allowed but might be filtered depending on permissions)."),
      '#default_value' => $text,
      '#required' => TRUE,
    );

    $form['conditional_message']['conditional_message_options_config'] = array(
      '#title' => t('Conditions configurations'),
      '#type' => 'fieldset',
      '#states' => array(
        'visible' => array(
          ':input[name="conditional_message_always"]' => array('checked' => FALSE),
          array(
            array(':input[name="conditional_message_options[session]"]' => array('checked' => TRUE)),
            array(':input[name="conditional_message_options[close]"]' => array('checked' => TRUE)),
            array(':input[name="conditional_message_options[role]"]' => array('checked' => TRUE)),
            array(':input[name="conditional_message_options[content_type]"]' => array('checked' => TRUE)),
            array(':input[name="conditional_message_options[path]"]' => array('checked' => TRUE)),
            array(':input[name="conditional_message_options[multisite]"]' => array('checked' => TRUE)),
          ),
        ),
      ),
    );

    $session_helper = '<label>About sessions:</label> Mind that sessions (localStorage) '
        . 'persist even after closing a tab or window. For testing purposes the '
        . '"session" can be cleared in a browser by typing the following command '
        . 'on the console: '
        . '"localStorage.removeItem(\'conditionalMessageReadStatus\');';
    $form['conditional_message']['conditional_message_options_config']['conditional_message_session'] = array(
      '#type' => 'container',
      '#children' => t($session_helper),
      '#states' => array(
        'visible' => array(
          ':input[name="conditional_message_options[session]"]' => array('checked' => TRUE),
        ),
      ),
    );

    $close_helper = '<label>About the close button:</label> The close button '
      . 'stores the action of clicking the "x" in the browser\'s localStorage. '
      . 'This information persists after closing the browser tab or window.'
      . 'To clear this info for testing purposes type the following command '
      . 'on the console: '
      . '"localStorage.removeItem(\'conditionalMessageUserClosed\');';
    $form['conditional_message']['conditional_message_options_config']['conditional_message_closed'] = array(
      '#type' => 'container',
      '#children' => t($close_helper),
      '#states' => array(
        'visible' => array(
          ':input[name="conditional_message_options[close]"]' => array('checked' => TRUE),
        ),
      ),
    );

    $form['conditional_message']['conditional_message_options_config']['conditional_message_user_role'] = array(
      '#type' => 'checkboxes',
      '#title' => t('User roles'),
      '#description' => t('Select roles that will see the message'),
      '#options' => user_roles(),
      '#default_value' => variable_get('conditional_message_user_role'),
      '#required' => FALSE,
      '#states' => array(
        'visible' => array(
          ':input[name="conditional_message_options[role]"]' => array('checked' => TRUE),
        ),
      ),
    );

    $form['conditional_message']['conditional_message_options_config']['conditional_message_content_type'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Content types'),
      '#description' => t('Select content types that will trigger the message'),
      '#options' => node_type_get_names(),
      '#default_value' => variable_get('conditional_message_content_type', array('article', 'page')),
      '#required' => FALSE,
      '#states' => array(
        'visible' => array(
          ':input[name="conditional_message_options[content_type]"]' => array('checked' => TRUE),
        ),
      ),
    );

    $form['conditional_message']['conditional_message_options_config']['conditional_message_path'] = array(
      '#type' => 'textarea',
      '#title' => t('Paths'),
      '#description' => t('Enter one path per line. Aliases are not considered as equivalent so node/1 and first-page are considered different.'),
      '#default_value' => variable_get('conditional_message_path'),
      '#required' => FALSE,
      '#states' => array(
        'visible' => array(
          ':input[name="conditional_message_options[path]"]' => array('checked' => TRUE),
        ),
      ),
    );

    $form['conditional_message']['conditional_message_options_config']['conditional_message_select'] = array(
      '#type' => 'button',
      '#value' => t('Check all multi-sites'),
      '#states' => array(
        'visible' => array(
          ':input[name="conditional_message_options[multisite]"]' => array('checked' => TRUE),
        ),
      ),
    );

    $form['conditional_message']['conditional_message_options_config']['conditional_message_unselect'] = array(
      '#type' => 'button',
      '#value' => t('Uncheck all multi-sites'),
      '#states' => array(
        'visible' => array(
          ':input[name="conditional_message_options[multisite]"]' => array('checked' => TRUE),
        ),
      ),
    );

    $form['conditional_message']['conditional_message_options_config']['conditional_message_multisite_client'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Multi-sites'),
      '#description' => t('Check the boxes of the multi-sites that will see the message. These sites must select the option "Get configuration from another multi-site"'),
      '#options' => conditional_message_multisites_list(),
      '#default_value' => variable_get('conditional_message_multisite_client', NULL),
      '#required' => FALSE,
      '#states' => array(
        'visible' => array(
          ':input[name="conditional_message_options[multisite]"]' => array('checked' => TRUE),
        ),
      ),
    );

    $form['conditional_message']['conditional_message_layout'] = array(
      '#title' => t('Message configurations'),
      '#type' => 'fieldset',
    );

    $form['conditional_message']['conditional_message_layout']['conditional_message_bg_color'] = array(
      '#title' => t('Background color'),
      '#type' => 'textfield',
      '#field_prefix' => '#',
      '#description' => 'Enter a color for the message background in HEX value.',
      '#default_value' => drupal_strtoupper(variable_get('conditional_message_bg_color', '119944')),
      '#size' => 6,
      '#maxlength' => 6,
      '#required' => FALSE,
    );

    $form['conditional_message']['conditional_message_layout']['conditional_message_color'] = array(
      '#title' => t('Font color'),
      '#type' => 'textfield',
      '#field_prefix' => '#',
      '#description' => 'Enter a color for the message text in HEX value.',
      '#default_value' => drupal_strtoupper(variable_get('conditional_message_color', 'EEEEEE')),
      '#size' => 6,
      '#maxlength' => 6,
      '#required' => FALSE,
    );

    $display = array(
      'top' => t('Top bar'),
      'bottom' => t('Bottom bar'),
  //    'overlay' => t('Overlay'),
    );
    $form['conditional_message']['conditional_message_layout']['conditional_message_display'] = array(
      '#type' => 'radios',
      '#title' => t('Message display'),
      '#default_value' => variable_get('conditional_message_display', 'top'),
      '#options' => $display,
      '#description' => t('Select the desired display for the conditional message.'),
    );

    $form['conditional_message']['conditional_message_layout']['conditional_message_target'] = array(
      '#title' => t('Target HTML container'),
      '#type' => 'textfield',
      '#description' => t('Enter an id, class or tag (any CSS selector) of the container where the message should be appended. If blank, "body" will be used.'),
      '#default_value' => variable_get('conditional_message_target', 'body'),
      '#required' => FALSE,
    );

    $form['#submit'][] = 'conditional_message_config_form_submit';
    $form['#validate'][] = 'conditional_message_config_form_validate';
    $form['#attached']['js'] = array(
      drupal_get_path('module', 'conditional_message') . '/js/conditional_message_form.js',
    );
  }

  return system_settings_form($form);
}

/**
 * Validation of the conditional message client configuration form.
 */
function conditional_message_client_config_form_validate($form, &$form_state) {
  $server_url = $form_state['values']['conditional_message_server_site'];
  if (empty($server_url)) {
    drupal_set_message(t('There is no server site selected. This option is required.'), 'warning');
  }
  else {
    $end = substr($server_url, -1, 1);
    if ($end == '/') {
      $form_state['values']['conditional_message_server_site'] = substr($server_url, 0, -1);
    }
  }
}

/**
 * Get list of multisites.
 */
function conditional_message_multisites_list() {
  // Gets a list of all directories in /sites.
  $sites = glob(DRUPAL_ROOT . '/sites/*', GLOB_ONLYDIR);
  // Strips the absolute path except for the last name.
  $sites = array_map('basename', $sites);
  // Removes the 'all' folder.
  $sites = array_diff($sites, array('all'));
  // Set keys equal to values to be able to identify sites later.
  $sites = array_values($sites);
  $sites = array_combine($sites, $sites);

  return $sites;
}

/**
 * Clear selected caches on form submit.
 */
function conditional_message_config_form_submit($form, &$form_state) {
  $selected_caches = variable_get('conditional_message_cache_options', ['']);
  // Drupal cache.
  if (in_array('drupal', $selected_caches)) {
    $url = url('/conditional_message_data_output');
    cache_clear_all($url, 'cache_page');
  }
  // Acquia cloud cache.
  if (module_exists('acquia_purge') && in_array('acquia', $selected_caches)) {
    $service = _acquia_purge_service();
    $service->addPath('conditional_message_data_output');
  }
}

/**
 * Validation of the conditional message configuration form.
 */
function conditional_message_config_form_validate($form, &$form_state) {
  // Verify if at least one condition is checked to display the message.
  if ($form_state['values']['conditional_message_always'] != 1) {
    $error = TRUE;
    foreach ($form_state['values']['conditional_message_options'] as $option) {
      if ($option !== 0) {
        $error = FALSE;
      }
    }
    if ($error) {
      drupal_set_message(t('There is no condition selected to trigger a message, it will never be displayed.'), 'warning');
    }
  }

  // Validate and save the message text.
  if (empty($form_state['values']['conditional_message_text']['value'])) {
    form_set_error('conditional_message_text', 'You need to provide a message to be displayed.');
  }
  else {
    $message_text = check_markup($form_state['values']['conditional_message_text']['value'], $form_state['values']['conditional_message_text']['format']);
    variable_set('conditional_message_text', $message_text);
  }

  // Check roles configurations, at least one selected.
  if ($form_state['values']['conditional_message_options']['role'] !== 0) {
    $error = TRUE;
    foreach ($form_state['values']['conditional_message_user_role'] as $value) {
      if ($value !== 0) {
        $error = FALSE;
      }
    }
    if ($error) {
      form_set_error('conditional_message_user_role', 'At least one role must be selected if you chose to use the role as a condition.');
    }
  }

  // Check path configuration consistency.
  if ($form_state['values']['conditional_message_options']['path'] !== 0) {
    if (empty(trim(str_replace(PHP_EOL, '', $form_state['values']['conditional_message_path'])))) {
      form_set_error('conditonal_message_path', 'Enter at least one path if you chose to use the path as a condition.');
    }
    // Check if the specified paths exists and are accessible.
    $paths = explode(PHP_EOL, $form_state['values']['conditional_message_path']);
    if (!is_array($paths)) {
      $paths = [$paths];
    }
    foreach ($paths as $raw_path) {
      $source_path = drupal_lookup_path('source', trim(str_replace(PHP_EOL, '', $raw_path)));
      $path = $source_path ? $source_path : $raw_path;
      // Empty lines are tolerated so we check if path is not empty.
      if (!empty($path) && !drupal_valid_path($path)) {
        form_set_error('conditonal_message_path', 'All paths must exist and you must have permission to access it.');
      }
    }
  }

  // Check content type config consistency.
  if ($form_state['values']['conditional_message_options']['content_type'] !== 0) {
    $error = TRUE;
    foreach ($form_state['values']['conditional_message_content_type'] as $value) {
      if ($value !== 0) {
        $error = FALSE;
      }
    }
    if ($error) {
      form_set_error('conditional_message_content_type', 'At least one content type must be selected if you chose to use the content type as a condition.');
    }
  }

  // Set notice if no multi-site client is selected.
  if (!empty($form_state['values']['conditional_message_options']['multisite'])) {
    $error = TRUE;
    foreach ($form_state['values']['conditional_message_multisite_client'] as $key => $sites) {
      if ($sites !== 0) {
        $error = FALSE;
      }
    }
    if ($error == TRUE) {
      drupal_set_message(t('There is no multi-site selected, the message won\'t be displayed.'), 'notice');
    }
  }

  // Check for valid HEX background color code.
  $c = $form_state['values']['conditional_message_bg_color'];
  if (!ctype_xdigit($c) || (drupal_strlen($c) !== 6 && drupal_strlen($c) !== 3)) {
    form_set_error('conditional_message_bg_color', 'Not a valid HEX color. Please enter a 3 or 6 HEX digit on the background color field.');
  }

  // Check for valid HEX font color code.
  $c = $form_state['values']['conditional_message_color'];
  if (!ctype_xdigit($c) || (drupal_strlen($c) !== 6 && drupal_strlen($c) !== 3)) {
    form_set_error('conditional_message_color', 'Not a valid HEX color. Please enter a 3 or 6 HEX digit on the font color field.');
  }

  // If the selector is blank, use body tag.
  if (empty($form_state['values']['conditional_message_target'])) {
    $form_state['values']['conditional_message_target'] = 'body';
  }
}
